﻿using UnityEngine;
using System.Collections;

public class TerrainGeneration : MonoBehaviour {

    private Terrain m_terrain;
    public TerrainData TerrainData;
    private float[,] m_heights;
    private int m_heightMapResolution;

    public int GridSize = 5;
    public float Variance = 0.02f;
    public float MaxHeight = 0.2f;
    public int Seed = 0;
    public bool ZeroCorners = false;
    public bool BoxifyHeights = false;
    public float BoxificationResolution = 0.002f;
    public bool SmoothHeights = false;
    public int SmoothIterations = 1;

    void OnEnable()
    {
        if (Seed != 0)
        {
            Random.seed = Seed;
        }

        m_terrain = transform.GetComponent<Terrain>();
        TerrainData = m_terrain.terrainData;

        m_heightMapResolution = (int)Mathf.Pow(2, GridSize) + 1;
        TerrainData.heightmapResolution = m_heightMapResolution;

        int terrainWidth = m_heightMapResolution - 1;
        TerrainData.size.Set(terrainWidth, terrainWidth, 250);


        m_heights = TerrainData.GetHeights(0, 0, m_heightMapResolution, m_heightMapResolution);

        if (ZeroCorners)
        {
            m_heights[0, 0] = 0.01f;
            m_heights[0, m_heightMapResolution - 1] = 0.01f;
            m_heights[m_heightMapResolution - 1, 0] = 0.01f;
            m_heights[m_heightMapResolution - 1, m_heightMapResolution - 1] = 0.01f;
        }
        else
        {
            m_heights[0, 0] = MaxHeight * Random.value;
            m_heights[0, m_heightMapResolution - 1] = MaxHeight * Random.value;
            m_heights[m_heightMapResolution - 1, 0] = MaxHeight * Random.value;
            m_heights[m_heightMapResolution - 1, m_heightMapResolution - 1] = MaxHeight * Random.value;
        }

        DiamondSqaure(0, 0, m_heightMapResolution - 1, GridSize);

        if (SmoothHeights)
        {
            for (int i = 0; i < SmoothIterations; i++)
            {
                for (int x = 0; x < m_heightMapResolution; x++)
                {
                    for (int z = 0; z < m_heightMapResolution; z++)
                    {
                        float n = 1;
                        float sum = m_heights[x, z];

                        if (x < m_heightMapResolution - 2)
                        {
                            sum += m_heights[x + 1, z];
                            n++;
                        }
                        if (x > 0)
                        {
                            sum += m_heights[x - 1, z];
                            n++;
                        }
                        if (z < m_heightMapResolution - 2)
                        {
                            sum += m_heights[x, z + 1];
                            n++;
                            if (x < m_heightMapResolution - 1)
                            {
                                sum += m_heights[x + 1, z + 1];
                                n++;
                            }
                            if (x > 0)
                            {
                                sum += m_heights[x - 1, z + 1];
                                n++;
                            }
                        }
                        if (z > 0)
                        {
                            sum += m_heights[x, z - 1];
                            n++;
                            if (x < m_heightMapResolution - 1)
                            {
                                sum += m_heights[x + 1, z - 1];
                                n++;
                            }
                            if (x > 0)
                            {
                                sum += m_heights[x - 1, z - 1];
                                n++;
                            }
                        }

                        m_heights[x, z] = sum / n;
                    }
                }
            }
        }

        if (BoxifyHeights)
        {
            for (int x = 0; x < m_heightMapResolution; x++)
            {
                for (int z = 0; z < m_heightMapResolution; z++)
                {
                    m_heights[x, z] -= m_heights[x, z] % BoxificationResolution;
                }
            }
        }

        TerrainData.SetHeights(0, 0, m_heights);
    }

    void DiamondSqaure(int x, int z, int s, int k)
    {
        m_heights[x + s / 2, z + s / 2] = (m_heights[x, z] + m_heights[x + s, z] + m_heights[x, z + s] + m_heights[x + s, z + s]) / 4 + RandomFactor(k);
            
        m_heights[x + s / 2, z] = (m_heights[x, z] + m_heights[x + s, z]) / 2;
        m_heights[x, z + s / 2] = (m_heights[x, z] + m_heights[x, z + s]) / 2;
        m_heights[x + s / 2, z + s] = (m_heights[x, z + s] + m_heights[x + s, z + s]) / 2;
        m_heights[x + s, z + s / 2] = (m_heights[x + s, z] + m_heights[x + s, z + s]) / 2;
            
        k--;

        if (k > 0)

        {
            DiamondSqaure(x, z, s / 2, k);
            DiamondSqaure(x + s / 2, z, s / 2, k);
            DiamondSqaure(x, z + s / 2, s / 2, k);
            DiamondSqaure(x + s / 2, z + s / 2, s / 2, k);
        }
    }

    float RandomFactor(int k)
    {
        return (MaxHeight * Random.value - 0.5f * MaxHeight) * Variance * k;
    }
}
